


class KinectBlobs
{
  // Images
  PImage depth;
  PImage depthTh;
  PGraphics depthThResize;

  // Blob detection
  BlobDetection bd;

  // Controls
  ControlP5 controls;
  ControlWindow controlWindow;

  // OSC
  OscP5 oscP5;
  NetAddress myRemoteLocation;
  String myRemoteIP = "192.168.0.1";
  boolean useOSC=false;

  // Parameters
  int thMin = 0;
  int thMax = 255;
  float areaMax = 500;
  int sizeFactor = 3;

  // Position
  float X=0,Y=0;

  // Parent
  PApplet p;

  KinectBlobs(PApplet parent)
  {
    p = parent;
  }

  void init()
  {
    NativeKinect.init();
    depth   = createImage(640,480,RGB);
    depthTh = createImage(depth.width,depth.height,RGB);
    depthThResize = createGraphics(64*sizeFactor,48*sizeFactor,JAVA2D);

    bd = new BlobDetection(depthThResize.width,depthThResize.height);
    bd.setPosDiscrimination(false);
    bd.setThreshold(200);
    bd.activeCustomFilter(p);

    initControls();
    oscInit();
  }

  void update()
  {
    NativeKinect.update();
    depth.pixels = NativeKinect.getDepthMap();
    depth.updatePixels();

    int nb=depth.width*depth.height;
    int p;
    int g;
    for (int i=0;i<nb;i++)
    {
      p = depth.pixels[i]; // ARGB
      g = p&0x000000FF; 
      if ( g <= thMin)
        depthTh.pixels[i] = 0;
      else
        if ( g >= thMax)
          depthTh.pixels[i] = 0;
        else     
          depthTh.pixels[i] = 0x00FFFFFF;
    }
    depthTh.updatePixels();

    depthThResize.beginDraw();
    depthThResize.image(depthTh,0,0,depthThResize.width,depthThResize.height);
    depthThResize.endDraw();

    bd.computeBlobs(depthThResize.pixels);

    // Send blobs if any
    if (bd.getBlobNb() > 0)
    {
      // First blob only
      Blob b = bd.getBlob(0);
      X = b.x*width;
      Y = b.y*height;

      sendBlob(b.x,b.y);
    }
  }

  void draw()
  {
    // Draw
    tint(255,255,255,60);
    image(depth,0,0,width,height);
    tint(255,100,255,60);
    image(depthThResize,0,0,width,height);
    
    drawBlobs();
  }

  void drawCursor()
  {
    float l = 10;
    line(X-l,Y,X+l,Y);
    line(X,Y-l,X,Y+l);
  }

  void oscInit()
  {
    if (useOSC) {
      oscP5 = new OscP5(this,12000);
      myRemoteLocation = new NetAddress(myRemoteIP,12000);
    }
  }

  void sendBlob(float x, float y)
  {
    if (useOSC)
    {
      OscMessage myMessage = new OscMessage("/blob");
      myMessage.add(x);
      myMessage.add(y);
      oscP5.send(myMessage, myRemoteLocation);
    }
  }

  void initControls()
  {
    controls = new ControlP5(p);
    controls.setAutoInitialization(true);
    controls.load("controlP5.xml");
    controls.setAutoDraw(false);
  }

  void drawBlobs() {
    pushStyle();
    stroke(255,0,0);
    rectMode(CENTER);

    Blob b;
    for (int n=0 ; n<bd.getBlobNb() ; n++)
    {
      b=bd.getBlob(n);
      noFill();
      rect(b.x*width,b.y*height,b.w*width,b.h*height);
      fill(255,0,0);
      textSize(12);
      text( nf(b.x,1,3) + "-" + nf(b.y,1,3), b.x*width,b.y*height);
    }
    popStyle();
  }
}


boolean newBlobDetectedEvent (Blob b)
{
  float w = b.w*width;
  float h = b.h*height;
  //println("a="+(w*h));
  if (w*h <= kb.areaMax)
    return false;
  return true;
}

void controlEvent(ControlEvent theEvent) {
  switch(theEvent.controller().id()) {
    case(1):
    kb.thMin = (int)(theEvent.controller().value());
    break;
    case(2):
    kb.thMax = (int)(theEvent.controller().value());
    break;
    case(3):
    kb.areaMax = (float)(theEvent.controller().value());
    //    println(kb.areaMax);
    break;
  }
}

