/*
  MetaSketch_ESA_Pau
  31 Janvier - 4 Février 2011 
  http://www.esac-pau.fr/

  Programmation
   - Julien Gachadoat ( www.v3ga.net )
   - David C.

  Sketches
   - Charline, Jessie, Marie-Charolotte, Marion, Mélaine, Samuel, Virginie
   
  Librairies
   - Kinect par Daniel Shiffman ( http://www.shiffman.net/2010/11/14/kinect-and-processing/ )
   - Geomerative par Ricard Marxer ( http://www.ricardmarxer.com/geomerative/ )
   - controlP5 par Andreas Schlegel ( http://www.sojamo.de/libraries/controlP5/ )
   - BlobDetection par Julien Gachadoat ( http://www.v3ga.net/processing/BlobDetection/ )

*/



import oscP5.*;
import netP5.*;
import controlP5.*;
import shiffman.kinect.*;
import blobDetection.*;
import ddf.minim.*;
import geomerative.*;

PFont introFont;

boolean displayIntro;
boolean displayKinect=false;
int smouseX, smouseY;
int psmouseX, psmouseY;
Sketch sketch;
Minim minim;
AudioInput in;
KinectBlobs kb;
boolean skip=false;
PApplet theApplet;

int currentSketch;
int timer;
int sketchDuration;
int introDuration;
boolean clearScreenAfterIntro = true;
Sketch[] sketchs;

void setup() {
  noCursor();

  theApplet = this;

  introFont = loadFont("Helvetica-90.vlw");

  currentSketch = 0;
  sketchDuration = 40 * 1000;
  introDuration = 15 * 100;
  timer = millis();

  kb = new KinectBlobs(this);
  kb.init();
  RG.init(this);
  minim = new Minim(this);
  in = minim.getLineIn(Minim.STEREO, 512);

  size(1024,768/*,JAVA2D*/);

  int id=0;
  sketchs = new Sketch[13];
  sketchs[id++] = new Jessie_2(in);
  sketchs[id++] = new Charlotte_2(in);
  sketchs[id++] = new Virginie_1(in);
  sketchs[id++] = new Charline_4(in);
  sketchs[id++] = new Samuel_1(in);
  sketchs[id++] = new Melaine_2(in);
  sketchs[id++] = new Marion_1(in);
  sketchs[id++] = new Virginie_2(in);
  sketchs[id++] = new Charline_2(in);
  sketchs[id++] = new Jessie_1(in);
  sketchs[id++] = new Melaine_1(in);
  sketchs[id++] = new Charlotte_1(in);
  sketchs[id++] = new Charline_3(in);

  for (int i=0;i<sketchs.length;i++)
    sketchs[i].data();

  sketchs[0].setup();
}

void draw() {
  kb.update();

  psmouseX = smouseX;
  psmouseY = smouseY;  
  //  smouseX = width-int(kb.X);
  smouseX =width -  int(kb.X);
  smouseY = int(kb.Y);  

  // Sketch switch
  if(skip || millis() - timer > sketchDuration+introDuration) {
    sketchs[currentSketch].exit();
    skip=false;
    timer = millis();
    currentSketch = (currentSketch+1)%sketchs.length;
    sketchs[currentSketch].setup();
    clearScreenAfterIntro = true;
  }
  else
  if(millis() - timer < introDuration) {
    if(!sketchs[currentSketch].intro()) {
      sketchs[currentSketch].draw();
    }
  } 
  else {
    if (clearScreenAfterIntro)
    {
      background(255);
      clearScreenAfterIntro = false;
    }

    sketchs[currentSketch].draw();
  }


  if(displayKinect)
    kb.draw();
}

void keyPressed() {
  if(keyCode == RIGHT) {
    skip=true;
  }

  if(key == 'i' || key == 'I') {
    displayIntro = !displayIntro;
  }
  if(key == 'k' || key == 'K') {
    displayKinect = !displayKinect;
  }
  if(key == 's' || key == 'S') {
    kb.controls.save("controlP5.xml");
  }
}


void stop()
{
  in.close();
  minim.stop();
  super.stop();
}

